#pragma once

//  $Header:
//------------------------------------------------------------------------------------------------
//
//  FILE:    CvXMLLoadUtility.h
//
//  AUTHOR:  Eric MacDonald  --  8/2003
//
//  PURPOSE: Group of functions to load in the xml files for Civilization 4
//
//------------------------------------------------------------------------------------------------
//  Copyright (c) 2003 Firaxis Games, Inc. All rights reserved.
//------------------------------------------------------------------------------------------------
#ifndef XML_LOAD_UTILITY_H
#define XML_LOAD_UTILITY_H

//#include "CvStructs.h"
#include "CvInfos.h"
#include "CvGlobals.h"

class FXmlSchemaCache;
class FXml;
class CvGameText;
class CvCacheObject;

//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//
//  class : CvXMLLoadUtility
//
//  DESC:   Group of functions to load in the xml files for Civilization 4
//
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
class CvXMLLoadUtility
{
//---------------------------------------PUBLIC INTERFACE---------------------------------
public:
	// default constructor
	DllExport CvXMLLoadUtility();
	// default destructor
	DllExport ~CvXMLLoadUtility(void);

	DllExport bool CreateFXml();
	DllExport void DestroyFXml();

	DllExport bool LoadPostMenuGlobals();
	DllExport bool LoadPreMenuGlobals();
	DllExport bool LoadBasicInfos();
	DllExport bool LoadPlayerOptions();
	DllExport bool LoadGraphicOptions();

	// read the global defines from a specific file
	DllExport bool ReadGlobalDefines(const TCHAR* szXMLFileName, CvCacheObject* cache);
	// loads globaldefines.xml and calls various other functions to load relevant global variables
	DllExport bool SetGlobalDefines();
	// loads globaltypes.xml and calls various other functions to load relevant global variables
	DllExport bool SetGlobalTypes();
	// loads calls various functions that load xml files that in turn load relevant global variables
	DllExport bool SetGlobals();
	// loads globaldefines.xml and calls various other functions to load relevant global variables
	DllExport bool SetPostGlobalsGlobalDefines();

	// calls various functions to release the memory associated with the global variables
	DllExport void CleanUpGlobalVariables();

	// releases global variables associated with items that can be reloaded
	DllExport void ResetLandscapeInfo();
	DllExport bool SetupGlobalLandscapeInfo();
	DllExport bool SetGlobalArtDefines();
	DllExport bool LoadGlobalText();
	DllExport bool SetHelpText();
	DllExport void ResetGlobalEffectInfo();

// for progress bars
	typedef void (*ProgressCB)(int iStepNum, int iTotalSteps, const char* szMessage);
	DllExport static int GetNumProgressSteps();
	DllExport void RegisterProgressCB(ProgressCB cbFxn) { m_pCBFxn = cbFxn; }

	// moves the current xml node from where it is now to the next non-comment node, returns false if it can't find one
	DllExport bool SkipToNextVal();

	DllExport void MapChildren();	// call this before GetChildXMLValByName to use fast searching
	// overloaded function that gets the child value of the tag with szName if there is only one child
	// value of that name, Obj-String versions
	// RH: Changed the string defaults to const - eases a few things
	DllExport bool GetChildXmlValByName(std::string& pszVal, const TCHAR* szName, const char* pszDefault = NULL);
	DllExport bool GetChildXmlValByName(std::wstring& pszVal, const TCHAR* szName, const wchar* pszDefault = NULL);
	// overloaded function that gets the child value of the tag with szName if there is only one child
	// value of that name, C-String versions
	DllExport bool GetChildXmlValByName(char* pszVal, const TCHAR* szName, const char* pszDefault = NULL);
	DllExport bool GetChildXmlValByName(wchar* pszVal, const TCHAR* szName, const wchar* pszDefault = NULL);
	// overloaded function that gets the child value of the tag with szName if there is only one child
	// value of that name, int version
	DllExport bool GetChildXmlValByName(int* piVal, const TCHAR* szName, int iDefault = 0);
	// overloaded function that gets the child value of the tag with szName if there is only one child
	// value of that name, float version
	DllExport bool GetChildXmlValByName(float* pfVal, const TCHAR* szName, float fDefault = 0.0f);
	// overloaded function that gets the child value of the tag with szName if there is only one child
	// value of that name, bool version
	DllExport bool GetChildXmlValByName(bool* pbVal, const TCHAR* szName, bool bDefault = false);
	// RH: A new set of overloads to ease implementation of the incremental modular XML feature
	// Note the bool version is missing - to avoid confusion and 'cause calling the existing function
	// passing the current value as default '&'-ed with the incremental flag is easy enough.
	DllExport bool GetChildXmlValByName(std::string& pszVal, const TCHAR* szName, bool bIncremental, const char* pszDefault = NULL);
	DllExport bool GetChildXmlValByName(std::wstring& pszVal, const TCHAR* szName, bool bIncremental, const wchar* pszDefault = NULL);
	DllExport bool GetChildXmlValByName(char* pszVal, const TCHAR* szName, bool bIncremental, const char* pszDefault = NULL);
	DllExport bool GetChildXmlValByName(wchar* pszVal, const TCHAR* szName, bool bIncremental, const wchar* pszDefault = NULL);
	DllExport bool GetChildXmlValByName(int* piVal, const TCHAR* szName, bool bIncremental, int iDefault = 0);
	DllExport bool GetChildXmlValByName(float* pfVal, const TCHAR* szName, bool bIncremental, float fDefault = 0.0f);
	// Version that translates InfoType Keys to int
	DllExport bool GetChildXmlKeyValByName(int* piVal, const TCHAR* szName, bool bIncremental);
	// RH: End

	// loads an xml file into the FXml variable.  The szFilename parameter has
	// the m_szXmlPath member variable pre-pended to it to form the full pathname
	DllExport bool LoadCivXml(FXml* pFXml, const TCHAR* szFilename);

	// overloaded function that gets either the current xml node's or the next non-comment xml node's string value
	// depending on if the current node is a non-comment node or not
	// RH: Changed the string defaults to const - eases a few things
	bool GetXmlVal(std::wstring& pszVal, const wchar* pszDefault = NULL);
	bool GetXmlVal(std::string& pszVal, const char* pszDefault = NULL);
	// overloaded function that gets either the current xml node's or the next non-comment xml node's string value
	// depending on if the current node is a non-comment node or not
	bool GetXmlVal(wchar* pszVal, const wchar* pszDefault = NULL);
	bool GetXmlVal(char* pszVal, const char* pszDefault = NULL);
	// overloaded function that gets either the current xml node's or the next non-comment xml node's int value
	// depending on if the current node is a non-comment node or not
	bool GetXmlVal(int* piVal, int iDefault = 0);
	// overloaded function that gets either the current xml node's or the next non-comment xml node's float value
	// depending on if the current node is a non-comment node or not
	bool GetXmlVal(float* pfVal, float fDefault = 0.0f);
	// overloaded function that gets either the current xml node's or the next non-comment xml node's boolean value
	// depending on if the current node is a non-comment node or not
	bool GetXmlVal(bool* pbVal, bool bDefault = false);

	// overloaded function that sets the current xml node to it's next sibling and then
	//	gets the next non-comment xml node's string value
	// RH: Changed the string defaults to const - eases a few things
	bool GetNextXmlVal(std::string& pszVal, const char* pszDefault = NULL);
	bool GetNextXmlVal(std::wstring& pszVal, const wchar* pszDefault = NULL);
	// overloaded function that sets the current xml node to it's next sibling and then
	//	gets the next non-comment xml node's string value
	bool GetNextXmlVal(char* pszVal, const char* pszDefault = NULL);
	bool GetNextXmlVal(wchar* pszVal, const wchar* pszDefault = NULL);
	// overloaded function that sets the current xml node to it's next sibling and then
	//	gets the next non-comment xml node's int value
	bool GetNextXmlVal(int* piVal, int iDefault = 0);
	// overloaded function that sets the current xml node to it's next sibling and then
	//	gets the next non-comment xml node's float value
	bool GetNextXmlVal(float* pfVal, float fDefault = 0.0f);
	// overloaded function that sets the current xml node to it's next sibling and then
	//	gets the next non-comment xml node's boolean value
	bool GetNextXmlVal(bool* pbVal, bool bDefault = false);

	// overloaded function that sets the current xml node to it's first non-comment child node
	//	and then gets that node's string value
	// RH: Changed the string defaults to const - eases a few things
	bool GetChildXmlVal(std::string& pszVal, const char* pszDefault = NULL);
	bool GetChildXmlVal(std::wstring& pszVal, const wchar* pszDefault = NULL);
	// overloaded function that sets the current xml node to it's first non-comment child node
	//	and then gets that node's string value
	bool GetChildXmlVal(char* pszVal, const char* pszDefault = NULL);
	bool GetChildXmlVal(wchar* pszVal, const wchar* pszDefault = NULL);
	// overloaded function that sets the current xml node to it's first non-comment child node
	//	and then gets that node's integer value
	bool GetChildXmlVal(int* piVal, int iDefault = 0);
	// overloaded function that sets the current xml node to it's first non-comment child node
	//	and then gets that node's float value
	bool GetChildXmlVal(float* pfVal, float fDefault = 0.0f);
	// overloaded function that sets the current xml node to it's first non-comment child node
	//	and then gets that node's boolean value
	bool GetChildXmlVal(bool* pbVal, bool bDefault = false);

#ifdef _USRDLL
	FXml* GetXML() { return m_pFXml; }
#endif

	// loads the local yield from the xml file
	int SetYields(int** ppiYield);

#ifdef _USRDLL
	template <class T>
	int SetCommerce(T** ppiCommerce);
#endif

	// allocate and set the feature struct variables for the CvBuildInfo class
	// RH: Expanded to take Incremental Modular XML into account
	void SetFeatureStruct(bool bIncremental, int** ppiFeatureTech, int** ppiFeatureTime, int** ppiFeatureProduction, bool** ppbFeatureRemove);
	// loads the improvement bonuses from the xml file
	// RH: Expanded to take Incremental Modular XML into account
	void SetImprovementBonuses(bool bIncremental, CvImprovementBonusInfo** ppImprovementBonus);

	// check through the pszList parameter for the pszVal and returns the location a match
	// is found if one is found
	static int FindInInfoClass(const TCHAR* pszVal, bool hideAssert = false);

#ifdef _USRDLL
	template <class T>
	void InitList(T **ppList, int iListLen, T val = 0);
#endif
	void InitStringList(CvString **ppszList, int iListLen, CvString szString);

	void InitImprovementBonusList(CvImprovementBonusInfo** ppImprovementBonus, int iListLen);
	// allocate and initialize the civilization's default buildings
	void InitBuildingDefaults(int **ppiDefaults);
	// allocate and initialize the civilization's default units
	void InitUnitDefaults(int **ppiDefaults);
	// allocate and initialize a 2 dimensional array of int pointers
	void Init2DIntList(int*** pppiList, int iSizeX, int iSizeY);
	// allocate and initialize a 2 dimensional array of float pointers
	void Init2DFloatList(float*** pppfList, int iSizeX, int iSizeY);
	// allocate and initialize a 2D array of DirectionTypes
	void Init2DDirectionTypesList(DirectionTypes*** pppiList, int iSizeX, int iSizeY);
	// allocate an array of int pointers
	void InitPointerIntList(int*** pppiList, int iSizeX);
	// allocate an array of float pointers
	void InitPointerFloatList(float*** pppfList, int iSizeX);

	// RH: The "allocate and initialize a list from a tag pair in the xml" function
	// SetVariableListTagPair now uses templates and allows the Incremental
	// Modular XML abilities with approriate parameters. The original signatures,
	// even though they specify parameters that are never used in the implementation,
	// are defined here as inline delegators for compatibility.
	//
	// While implementing the Incremental Modular XML feature, I moved SetVariableListTagPair
	// to template usage. Originally there were a number of explicitly written overloads...
	// a) Targetting arrays of int, bool, float or CvString
	// b) Using FindInInfoClass or GC.getTypesEnum to translate tags to indices
	//		(these differentiated by the third parameter being int or CvString*, but the parameter was not actually used)
	// c) A "...ForAudioScripts" variant translating string values to integers using gDLL->getAudioTagIndex
	// The actual implementation is now a templated overload with a distinct signature (first parameter bool)
	// that knowns how to supply all the functionality plus the Incremental case. The CvString target case
	// is not covered by the template (different InitList, different dereferencing), neither is ...ForAudioScripts.
	// 
	// The old signatures are provided for compatibility, their implementations delegate to the new one.
	// Note the template definition is in this header as well, since the compiler does not seem to publish
	// the resolved templates for the linker to use, and this is called from CvInfos.cpp as well.
	template <class T>
	void SetVariableListTagPairI(
			bool bIncremental, bool bUseEnums,
			T **ppxList, const TCHAR* szRootTagName,
			int iArrayLength, T xDefaultListVal = NULL);

	void SetVariableListTagPairI(
			bool bIncremental, bool bUseEnums,
			CvString **ppszList, const TCHAR* szRootTagName,
			int iArrayLength, CvString szDefaultListVal = "");

	void SetVariableListTagPairForAudioScriptsI(
			bool bIncremental, bool bUseEnums,
			int **ppiList, const TCHAR* szRootTagName,
			int iArrayLength, int iDefaultListVal = -1);

	// Delegators for the original SetVariableListTagPair overload signature using FindInInfoClass
	inline void SetVariableListTagPair(
			int **ppiList, const TCHAR* szRootTagName,
			int iInfoBaseSize, int iInfoBaseLength, int iDefaultListVal = 0)
	{
		SetVariableListTagPairI(false,false,ppiList,szRootTagName,iInfoBaseLength,iDefaultListVal);
	}
	inline void SetVariableListTagPair(
			bool **ppbList, const TCHAR* szRootTagName,
			int iInfoBaseSize, int iInfoBaseLength, bool bDefaultListVal = false)
	{
		SetVariableListTagPairI(false,false,ppbList,szRootTagName,iInfoBaseLength,bDefaultListVal);
	}
	inline void SetVariableListTagPair(
			float **ppfList, const TCHAR* szRootTagName,
			int iInfoBaseSize, int iInfoBaseLength, float fDefaultListVal = 0.0f)
	{
		SetVariableListTagPairI(false,false,ppfList,szRootTagName,iInfoBaseLength,fDefaultListVal);
	}
	inline void SetVariableListTagPair(
			CvString **ppszList, const TCHAR* szRootTagName,
			int iInfoBaseSize, int iInfoBaseLength, CvString szDefaultListVal = "")
	{
		SetVariableListTagPairI(false,false,ppszList,szRootTagName,iInfoBaseLength,szDefaultListVal);
	}


	// Delegators for the original SetVariableListTagPair overload signature using GC.getTypesEnum
	inline void SetVariableListTagPair(
			int **ppiList, const TCHAR* szRootTagName,
			CvString* m_paszTagList, int iTagListLength, int iDefaultListVal = 0)
	{
		SetVariableListTagPairI(false,true,ppiList,szRootTagName,iTagListLength,iDefaultListVal);
	}
	inline void SetVariableListTagPair(
			bool **ppbList, const TCHAR* szRootTagName,
			CvString* m_paszTagList, int iTagListLength, bool bDefaultListVal = false)
	{
		SetVariableListTagPairI(false,true,ppbList,szRootTagName,iTagListLength,bDefaultListVal);
	}
	inline void SetVariableListTagPair(
			float **ppfList, const TCHAR* szRootTagName,
			CvString* m_paszTagList, int iTagListLength, float fDefaultListVal = 0.0f)
	{
		SetVariableListTagPairI(false,true,ppfList,szRootTagName,iTagListLength,fDefaultListVal);
	}
	inline void SetVariableListTagPair(
			CvString **ppszList, const TCHAR* szRootTagName,
			CvString* m_paszTagList, int iTagListLength, CvString szDefaultListVal = "")
	{
		SetVariableListTagPairI(false,true,ppszList,szRootTagName,iTagListLength,szDefaultListVal);
	}

	// Delegator for the AudioScripts / FindInInfoClass case
	void SetVariableListTagPairForAudioScripts(int **ppiList, const TCHAR* szRootTagName,
		int iInfoBaseLength, int iDefaultListVal = -1)
	{
		SetVariableListTagPairForAudioScriptsI(false,false,ppiList,szRootTagName,iInfoBaseLength,iDefaultListVal);
	}

	// Delegator for the AudioScripts / GC.getTypesEnum case
	void SetVariableListTagPairForAudioScripts(int **ppiList, const TCHAR* szRootTagName,
		CvString* m_paszTagList, int iTagListLength, int iDefaultListVal = -1)
	{
		SetVariableListTagPairForAudioScriptsI(false,true,ppiList,szRootTagName,iTagListLength,iDefaultListVal);
	}

	//
	// RH: The above Set* expect pairs of structure
	// <element><element>InfoType_Key</element><element>Value</element></element>
	// There's cases where the "Value" is a InfoType key as well and they get translated and
	// put into an int array. The original code wrote a block using a temp string array and
	// translating loop (or fully expanded loops) each time, this I packaged into a helper routine.
	// The routine also deals with the bIncremental flag appropriately.
	void SetVariableListTagKeyPair(bool bIncremental, int **ppiList, const TCHAR* szRootTagName, int iInfoBaseLength, int iDefaultListVal = -1);
	// Another 'moved' code - build an int array or vector from a bunch of elements containing info type keys
	// Note there are no pairs as above, and the resulting indexes are simply stacked into the array/vector unsorted.
	// Note also the int** version *can* determine the array size dynamically from the number of nodes in the XML
	// by passing a -1 for iArrayLength (or an int pointer to get the actual length), the vector version always does this.
	// In the incremental case, the whole list is still replaced when the root tag is non-empty,
	// only a missing or empty root tag will leave the existing list alone.
	void SetVariableListKeyList(bool bIncremental, int **ppiList, const TCHAR* szRootTagName, int iArrayLength, int iDefaultListVal = -1, int * piArrayLength = NULL);
	inline void SetVariableListKeyList(bool bIncremental, int **ppiList, const TCHAR* szRootTagName, int* piArrayLength, int iDefaultListVal = -1) {
		SetVariableListKeyList(bIncremental, ppiList, szRootTagName, -1, iDefaultListVal, piArrayLength );
	}
	template <typename T>
	void SetVariableListKeyList(bool bIncremental, std::vector<T>& aList, const TCHAR* szRootTagName);
	// Build a string vector from an element containing a plain sequence of PCDATA-containing subelements
	template <class T>
	void SetVariableListPlainStrings(bool bIncremental, std::vector<T>& aList, const TCHAR* szRootTagName);
	// Same for a string array
	void SetVariableListPlainStrings(bool bIncremental, CvString **ppszList, const TCHAR* szRootTagName, int iArraySize);
	// Build a 2D Array, an InfoType enum as first dimension, Yields as second
	void Set2DArrayKeyYield(bool bIncremental, int*** pppiList, int iSizeX, const TCHAR* szRootTagName, const TCHAR* szElementTagName, const TCHAR* szKeyTagName, const TCHAR* szYieldTagName );
	// RH: End Incremental Modular XML additions

	// create a hot key from a description
	CvWString CreateHotKeyFromDescription(const TCHAR* pszHotKey, bool bShift = false, bool bAlt = false, bool bCtrl = false);

	// set the variable to a default and load it from the xml if there are any children
	bool SetAndLoadVar(int** ppiVar, int iDefault=0);

	// function that sets the number of strings in a list, initializes the string to the correct length, and fills it from the
	// current xml file, it assumes that the current node is the parent node of the string list children
	bool SetStringList(CvString** ppszStringArray, int* piSize);

	// get the integer value for the keyboard mapping of the hotkey if it exists
	int GetHotKeyInt(const TCHAR* pszHotKeyVal);

	//---------------------------------------PRIVATE MEMBER VARIABLES---------------------------------
private:
	FXml* m_pFXml;						// member variable pointer to the current FXml class
	FXmlSchemaCache* m_pSchemaCache;	// keep a single schema cache, instead of loading the same schemas multiple times
	int m_iCurProgressStep;
	ProgressCB m_pCBFxn;

//---------------------------------------PRIVATE INTERFACE---------------------------------
private:
	void UpdateProgressCB(const char* szMessage=NULL);

	// take a character string of hex values and return their unsigned int value
	void MakeMaskFromString(unsigned int *puiMask, char* szMask);

	// find the tag name in the xml file and set the string parameter and num val parameter based on it's value
	void SetGlobalStringArray(CvString** ppszString, char* szTagName, int* iNumVals, bool bUseEnum=false);
	void SetDiplomacyCommentTypes(CvString** ppszString, int* iNumVals);	// sets diplomacy comments

	void SetGlobalUnitScales(float* pfLargeScale, float* pfSmallScale, char* szTagName);

#ifdef _USRDLL
	template <class T>
		void SetGlobalDefine(const char* szDefineName, T*& piDefVal)
	{ GC.getDefinesVarSystem()->GetValue(szDefineName, piDefVal); }
#endif
	//
	// template which can handle all info classes
	//
	// a dynamic value for the list size
#ifdef _USRDLL
	template <class T>
	void SetGlobalClassInfo(std::vector<T*>& aInfos, const char* szTagName, bool bPassTwo, bool bIsModular = false);
	template <class T>
	void LoadGlobalClassInfo(std::vector<T*>& aInfos, const char* szFileRoot, const char* szFileDirectory, const char* szXmlPath, bool bTwoPass, CvCacheObject* (CvDLLUtilityIFaceBase::*pArgFunction) (const TCHAR*) = NULL);
#endif
	void SetDiplomacyInfo(std::vector<CvDiplomacyInfo*>& DiploInfos, const char* szTagName);
	void LoadDiplomacyInfo(std::vector<CvDiplomacyInfo*>& DiploInfos, const char* szFileRoot, const char* szFileDirectory, const char* szXmlPath, CvCacheObject* (CvDLLUtilityIFaceBase::*pArgFunction) (const TCHAR*));

	//
	// special cases of set class info which don't use the template because of extra code they have
	//
	void SetGlobalActionInfo();
	void SetGlobalAnimationPathInfo(CvAnimationPathInfo** ppAnimationPathInfo, char* szTagName, int* iNumVals);
	void SetGameText(const char* szTextGroup, const char* szTagName);

	// create a keyboard string from a KB code, Delete would be returned for KB_DELETE
	CvWString CreateKeyStringFromKBCode(const TCHAR* pszHotKey);

	void orderHotkeyInfo(int** ppiSortedIndex, int* pHotkeyIndex, int iLength);
	void logMsg(char* format, ... );
};

#ifdef _USRDLL
//
/////////////////////////// inlines / templates
//
template <class T>
void CvXMLLoadUtility::InitList(T **ppList, int iListLen, T val)
{
	SAFE_DELETE_ARRAY(*ppList);
	FAssertMsg((0 <= iListLen),"list size to allocate is less than 0");
	*ppList = new T[iListLen];

	for (int i=0;i<iListLen;i++)
		(*ppList)[i] = val;
}

template <class T>
int CvXMLLoadUtility::SetCommerce(T** ppbCommerce)
{
	int i=0;			//loop counter
	int iNumSibs=0;		// the number of siblings the current xml node has
	T *pbCommerce;	// local pointer for the Commerce memory

	// Skip any comments and stop at the next value we might want
	if (SkipToNextVal())
	{
		// get the total number of children the current xml node has
		iNumSibs = gDLL->getXMLIFace()->GetNumChildren(m_pFXml);
		InitList(ppbCommerce, NUM_COMMERCE_TYPES);

		pbCommerce = *ppbCommerce;
		if (0 < iNumSibs)
		{
			// if the call to the function that sets the current xml node to it's first non-comment
			// child and sets the parameter with the new node's value succeeds
			if (GetChildXmlVal(&pbCommerce[0]))
			{
				FAssertMsg((iNumSibs <= NUM_COMMERCE_TYPES) , "For loop iterator is greater than array size");
				// loop through all the siblings, we start at 1 since we already have the first value
				for (i=1;i<iNumSibs;i++)
				{
					if (!GetNextXmlVal(&pbCommerce[i]))
					{
						break;
					}
				}
				gDLL->getXMLIFace()->SetToParent(m_pFXml);
			}
		}
	}

	return iNumSibs;
}

// RH: Moved from cpp file and converted to template. See Declaration section.
//------------------------------------------------------------------------------------------------------
//
//  FUNCTION:   SetVariableListTagPair(	int **ppiList, const TCHAR* szRootTagName,
//										int iInfoBaseSize, int iInfoBaseLength, int iDefaultListVal)
//
//  PURPOSE :   allocate and initialize a list from a tag pair in the xml
//
//------------------------------------------------------------------------------------------------------
#include "CvDLLXMLIFaceBase.h"

template <typename T>
void CvXMLLoadUtility::SetVariableListTagPairI(
		bool bIncremental, bool bUseEnums,
		T **ppxList, const TCHAR* szRootTagName,
		int iArrayLength, T xDefaultListVal)
{
	int iIndexVal;
	TCHAR szTextVal[256];
	T* pxList;

	if (0 > iArrayLength) {
		char	szMessage[1024];
		sprintf( szMessage, "Allocating zero or less memory in CvXMLLoadUtility::SetVariableListTagPair \n Current XML file is: %s", GC.getCurrentXMLFile().GetCString());
		gDLL->MessageBox(szMessage, "XML Error");
		return;
	}
	if (!bIncremental) {
		InitList(ppxList, iArrayLength, xDefaultListVal);
	}
	if (gDLL->getXMLIFace()->SetToChildByTagName(m_pFXml,szRootTagName)) {
		if (SkipToNextVal()) {
			pxList = *ppxList;
			if (gDLL->getXMLIFace()->SetToChild(m_pFXml)) {
				while (true) {
					if (!GetChildXmlVal(szTextVal)) break;
					if (bUseEnums) {
						iIndexVal = GC.getTypesEnum(szTextVal);
					} else {
						iIndexVal = FindInInfoClass(szTextVal);
					}
					if (iIndexVal >= iArrayLength) {
						char szMessage[1024];
						sprintf( szMessage, "Index out of bounds in CvXMLLoadUtility::SetVariableListTagPair \n Current XML file is: %s", GC.getCurrentXMLFile().GetCString());
						gDLL->MessageBox(szMessage, "XML Error");
					} else if (iIndexVal >= 0) {
						GetNextXmlVal(&pxList[iIndexVal]);
					}
					gDLL->getXMLIFace()->SetToParent(m_pFXml);
					if (!gDLL->getXMLIFace()->NextSibling(m_pFXml)) break;
				}
				gDLL->getXMLIFace()->SetToParent(m_pFXml);
			}
		}
		gDLL->getXMLIFace()->SetToParent(m_pFXml);
	}
}

// Build a vector from a list of InfoType key elements.
// as the overload for int arrays (in the cpp), but builds a vector.
// Largely different structure from the sibling above because both are copied as verbatim as possible
// from original inline CvInfos.cpp code - which had these different approaches.
template <typename T>
void CvXMLLoadUtility::SetVariableListKeyList(bool bIncremental, std::vector<T>& aList, const TCHAR* szRootTagName)
{
	CvString szTextVal;
	bool bNextSibling;
	int iIndex;

	if (!bIncremental) {
		aList.clear();
	}
	if ( gDLL->getXMLIFace()->SetToChildByTagName(m_pFXml, szRootTagName ))
	{
		if ( gDLL->getXMLIFace()->SetToChild( m_pFXml ) )
		{
			GetXmlVal( szTextVal );
			do 
			{
				iIndex = FindInInfoClass(szTextVal);
				if ( iIndex != -1 )
					aList.push_back( (T)iIndex );
				bNextSibling = GetNextXmlVal( szTextVal );
			} 
			while( bNextSibling );
			gDLL->getXMLIFace()->SetToParent(m_pFXml);
		}
		gDLL->getXMLIFace()->SetToParent(m_pFXml);
	}
}

// Fetch a bunch of strings (each within an element, all these elements being siblings in a container element)
// into a vector of CvStrings or CvWStrings
template <class T>
void CvXMLLoadUtility::SetVariableListPlainStrings(bool bIncremental, std::vector<T>& aList, const TCHAR* szRootTagName)
{
	T szTextVal;

	if (!bIncremental)
		aList.clear();

	if (gDLL->getXMLIFace()->SetToChildByTagName(m_pFXml,szRootTagName)) {
		if (SkipToNextVal()) {
			int iNumSibs = gDLL->getXMLIFace()->GetNumChildren(m_pFXml);
			aList.clear();

			if (0 < iNumSibs) {
				if (GetChildXmlVal(szTextVal)) {
					for (int j=0; j<iNumSibs; ++j) {
						aList.push_back(szTextVal);
						if (!GetNextXmlVal(szTextVal)) {
							break;
						}
					}

					gDLL->getXMLIFace()->SetToParent(m_pFXml);
				}
			}
		}
		gDLL->getXMLIFace()->SetToParent(m_pFXml);
	}
}

// RH: End

#endif

#endif	// XML_LOAD_UTILITY_H
